@@ -22,7 +22,6 @@ use std::{
 
 use bitflags::bitflags;
 use libseccomp::ScmpSyscall;
-use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
     fcntl::{AtFlags, OFlag},
@@ -896,14 +895,14 @@ impl DirEntry<'_> {
         self.dirent
     }
 
-    /// Returns true if this is a dot entry.
-    /// The special dot entries are `.` and `..`
+    /// Returns true if this is the `.` (dot) entry.
     pub fn is_dot(&self) -> bool {
-        if !self.is_dir() {
-            return false;
-        }
-        let name = self.name_bytes();
-        is_equal(name, b"..") || is_equal(name, b".")
+        self.is_dir() && self.as_xpath().is_equal(b".")
+    }
+
+    /// Returns true if this is the `..` (dotdot) entry.
+    pub fn is_dotdot(&self) -> bool {
+        self.is_dir() && self.as_xpath().is_equal(b"..")
     }
 
     /// Returns true if this is a directory entry.
