@@ -41,10 +41,10 @@ use serde::Serialize;
 use crate::{
     caps,
     compat::{waitid, WaitStatus},
-    config::{FADVISE_SYSCALLS, HAVE_RWF_NOAPPEND, MMAP_MIN_ADDR, UNSAFE_PERSONAS},
+    config::{FADVISE_SYSCALLS, HAVE_RWF_NOAPPEND, MMAP_MIN_ADDR, SYD_MADVISE, UNSAFE_PERSONAS},
     cookie::safe_socket,
     err::{err2no, SydResult},
-    fs::{nlmsg_align, readlinkat, retry_on_eintr, safe_clone, FileType, AT_BADFD},
+    fs::{nlmsg_align, readlinkat, safe_clone, FileType, AT_BADFD},
     info,
     landlock::{
         path_beneath_rules, Access, AccessFs, AccessNet, NetPort, RestrictSelfFlags,
@@ -52,6 +52,7 @@ use crate::{
         Scope, ABI,
     },
     path::{mask_path, XPathBuf},
+    retry::retry_on_eintr,
     sandbox::{IoctlMap, Sandbox},
 };
 
@@ -179,7 +180,7 @@ pub fn confine_scmp_wx() -> SydResult<()> {
     }
 
     if let Ok(syscall) = ScmpSyscall::from_name("personality") {
-        #[allow(clippy::cast_sign_loss)]
+        #[expect(clippy::cast_sign_loss)]
         for persona in UNSAFE_PERSONAS {
             let persona = persona.bits() as u64;
             ctx.add_rule_conditional(
@@ -235,7 +236,7 @@ pub fn confine_scmp_ioctl(denylist: &IoctlMap, ssb: bool) -> SydResult<()> {
         ctx.remove_arch(ScmpArch::native())?;
         ctx.add_arch(*arch)?;
 
-        #[allow(clippy::useless_conversion)]
+        #[expect(clippy::useless_conversion)]
         for request in denylist {
             let request = (*request).into();
             ctx.add_rule_conditional(
@@ -291,7 +292,7 @@ pub fn confine_scmp_pwritev2(ssb: bool) -> SydResult<()> {
         ctx.set_ctl_nnp(true)?;
 
         // Disable Speculative Store Bypass mitigations
-        // with trace/allow_unsafe_spec_exec:1
+        // with trace/allow_unsafe_exec_speculative:1
         ctx.set_ctl_ssb(ssb)?;
 
         // Synchronize filter to all threads.
@@ -328,7 +329,7 @@ pub fn confine_scmp_pwritev2(ssb: bool) -> SydResult<()> {
         ctx.set_ctl_nnp(true)?;
 
         // Disable Speculative Store Bypass mitigations
-        // with trace/allow_unsafe_spec_exec:1
+        // with trace/allow_unsafe_exec_speculative:1
         ctx.set_ctl_ssb(ssb)?;
 
         // Do not synchronize filter to all threads.
@@ -383,7 +384,7 @@ pub fn confine_scmp_clone(ctx: &mut ScmpFilterContext) -> SydResult<()> {
         | CloneFlags::CLONE_NEWPID
         | CloneFlags::CLONE_NEWCGROUP
         | CLONE_NEWTIME;
-    #[allow(clippy::cast_sign_loss)]
+    #[expect(clippy::cast_sign_loss)]
     let ns_mask = ns_mask.bits() as u64;
 
     // On s390/s390x the first two parameters to clone are switched.
@@ -479,6 +480,20 @@ pub fn confine_scmp_fadvise(ctx: &mut ScmpFilterContext) -> SydResult<()> {
     Ok(())
 }
 
+/// Confine madvise(2) advice.
+pub fn confine_scmp_madvise(ctx: &mut ScmpFilterContext) -> SydResult<()> {
+    if let Ok(syscall) = ScmpSyscall::from_name("madvise") {
+        for advice in SYD_MADVISE {
+            ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg2 == *advice)])?;
+        }
+    } else {
+        info!("ctx": "confine", "op": "allow_syscall",
+            "msg": "invalid or unsupported syscall madvise");
+    }
+
+    Ok(())
+}
+
 /// Convenience `Command` run wrapper which returns:
 ///
 /// - Same exit code as the process on clean exit.
@@ -777,7 +792,7 @@ impl SydMsFlags {
     }
 
     /// Convert MsFlags to a vector of flag names
-    #[allow(clippy::cognitive_complexity)]
+    #[expect(clippy::cognitive_complexity)]
     pub(crate) fn to_names(&self) -> Vec<&str> {
         let mut names = Vec::new();
 
@@ -861,40 +876,50 @@ impl SydMsFlags {
 #[inline]
 pub(crate) fn op2name(op: u8) -> &'static str {
     match op {
+        0x1 => "socket",
         0x2 => "bind",
         0x3 => "connect",
         0x5 => "accept",
         0x6 => "getsockname",
+        0x7 => "getpeername",
+        0x8 => "socketpair",
+        0x9 => "send",
         0xb => "sendto",
+        0xc => "recvfrom",
+        0xf => "getsockopt",
         0x10 => "sendmsg",
+        0x11 => "recvmsg",
         0x12 => "accept4",
+        0x13 => "recvmmsg",
+        u8::MAX => "recvmmsg_time64",
         0x14 => "sendmmsg",
         _ => unreachable!("BUG: op2name called with unsupported op:{op:#x}, report a bug!"),
     }
 }
 
-#[inline]
 pub(crate) fn op2errno(op: u8) -> Errno {
+    // We used to return:
+    // 0x2 = bind = EADDRNOTAVAIL.
+    // 0x3 = connect = ECONNREFUSED.
+    // 0xb,0x10,0x14 = send{to,{m,}msg} = ENOTCONN.
+    // As of version 3.41.8, this was changed to EACCES.
+    // This is consistent with SELinux.
+    // EACCES is not a valid errno(3) for accept{,4}.
+    // EACCES is not a valid errno(3) for socketpair(2).
     match op {
-        0x2 /*bind*/ => Errno::EADDRNOTAVAIL,
-        0x3 /*connect*/ =>  Errno::ECONNREFUSED,
         0x5 | 0x12 /*accept{,4}*/ => Errno::ECONNABORTED,
-        _ /*send{to,{m,}msg}*/ => Errno::ENOTCONN,
+        0x8 /* socketpair */ => Errno::EOPNOTSUPP,
+        _ => Errno::EACCES,
     }
 }
 
 /// Checks if the given namespaces are enabled.
 pub fn ns_enabled(ns_flags: CloneFlags) -> Result<bool, Errno> {
-    const SAFE_CLONE_FLAGS: libc::c_int = libc::CLONE_FS
-        | libc::CLONE_FILES
-        | libc::CLONE_IO
-        | libc::CLONE_VM
-        | libc::CLONE_VFORK
-        | libc::CLONE_SIGHAND;
+    // CLONE_SIGHAND|CLONE_V{FORK,M} are not included intentionally.
+    const SAFE_CLONE_FLAGS: libc::c_int = libc::CLONE_FS | libc::CLONE_FILES | libc::CLONE_IO;
 
     // All set, spawn the thread to check unprivileged userns.
     let mut stack = [0u8; crate::config::MINI_STACK_SIZE];
-    #[allow(clippy::blocks_in_conditions)]
     let pid_fd = safe_clone(
         Box::new(|| -> isize {
             if unshare(ns_flags).is_ok() {
@@ -935,7 +960,6 @@ pub fn lock_enabled(abi: ABI) -> u8 {
     };
 
     // A helper function to wrap the operations and reduce duplication
-    #[allow(clippy::disallowed_methods)]
     fn landlock_operation(
         abi: ABI,
         path_ro: &[XPathBuf],
@@ -980,7 +1004,7 @@ pub fn lock_enabled(abi: ABI) -> u8 {
 }
 
 /// Returns true if we are running under syd.
-#[allow(clippy::disallowed_methods)]
+#[expect(clippy::disallowed_methods)]
 pub fn syd_enabled() -> bool {
     // This will not work if the sandbox is locked.
     // Path::new("/dev/syd").exists() || Path::new("/dev/syd").exists()
@@ -1296,7 +1320,7 @@ impl ScmpNotifReq {
 
     #[inline(always)]
     pub(crate) fn pid(&self) -> Pid {
-        #[allow(clippy::cast_possible_wrap)]
+        #[expect(clippy::cast_possible_wrap)]
         Pid::from_raw(self.pid as libc::pid_t)
     }
 }
@@ -1369,7 +1393,7 @@ pub const fn scmp_arch_raw(arch: ScmpArch) -> u32 {
 }
 
 /// Confine creation of the given file type using mknod(2) and mknodat(2).
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn scmp_add_mknod(
     ctx: &mut ScmpFilterContext,
     action: ScmpAction,
@@ -1398,7 +1422,7 @@ pub(crate) fn scmp_add_mknod(
 }
 
 /// Add UID/GID change rules for SafeSetId.
-#[allow(clippy::cognitive_complexity)]
+#[expect(clippy::cognitive_complexity)]
 pub(crate) fn scmp_add_setid_rules(
     tag: &str,
     ctx: &mut ScmpFilterContext,
@@ -1829,8 +1853,8 @@ pub fn has_vdso_symbol(sym: &CStr) -> bool {
 }
 
 /// Check support for CONFIG_UNIX_DIAG in Linux kernel.
-#[allow(clippy::arithmetic_side_effects)]
-#[allow(clippy::cast_possible_truncation)]
+#[expect(clippy::arithmetic_side_effects)]
+#[expect(clippy::cast_possible_truncation)]
 pub fn check_unix_diag() -> Result<bool, Errno> {
     const SOCK_DIAG_BY_FAMILY: u16 = 20;
     const NL_HDR_LEN: usize = 16;
@@ -2010,7 +2034,7 @@ pub fn check_fd_leaks(fd_max: Option<RawFd>) -> u32 {
         // Create a PathBuf from the string representation of the file descriptor
         let link_path = proc_fd_path.join(fd_str.into_owned()); // Convert Cow<str> into a String and then into a PathBuf
 
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         match std::fs::read_link(&link_path) {
             Ok(target_path) => {
                 eprintln!("!!! Leaked file descriptor {fd} -> {target_path:?} !!!");
@@ -2133,7 +2157,7 @@ pub fn safe_drop_cap(cap: caps::Capability) -> Result<(), caps::errors::CapsErro
 
 /// Return true if the given signal has default action Core.
 #[inline]
-#[allow(unreachable_patterns)]
+#[expect(unreachable_patterns)]
 pub(crate) fn is_coredump(sig: i32) -> bool {
     matches!(
         sig,
@@ -2175,7 +2199,7 @@ impl FromStr for ExportMode {
 
 impl ExportMode {
     /// Return the export mode specified by the environment.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     pub fn from_env() -> Option<ExportMode> {
         Self::from_str(&std::env::var(crate::config::ENV_DUMP_SCMP).ok()?).ok()
     }
