@@ -10,7 +10,6 @@ macro_rules! syscall_handler {
     ($request:ident, $body:expr) => {{
         let request_id = $request.scmpreq.id;
 
-        #[expect(clippy::arithmetic_side_effects)]
         match $body($request) {
             Ok(result) => result,
             // SAFETY: Harden against UnknownErrno so as not to
@@ -19,7 +18,11 @@ macro_rules! syscall_handler {
             // SAFETY: ECANCELED is used by Syd internally to denote
             // requests that should be turned into no-op.
             Err(Errno::ECANCELED) => ScmpNotifResp::new(request_id, 0, 0, 0),
-            Err(errno) => ScmpNotifResp::new(request_id, 0, -(errno as i32), 0),
+            Err(errno) => {
+                // `ScmpNotifResp` expects negated errno.
+                let errno = (errno as i32).checked_neg().unwrap_or(-libc::ENOSYS);
+                ScmpNotifResp::new(request_id, 0, errno, 0)
+            }
         }
     }};
 }
@@ -129,6 +132,9 @@ pub(crate) mod unlink;
 /// xattr handlers
 pub(crate) mod xattr;
 
+/// ptrace(2) hooks
+pub(crate) mod ptrace;
+
 use std::borrow::Cow;
 
 use libseccomp::ScmpNotifResp;
