@@ -30,7 +30,7 @@ use crate::{
         SEE_EMILY_PLAY, SYD_EL, THE_PIPER,
     },
     confine::EIDRM,
-    cookie::{safe_memfd_create, safe_openat2},
+    cookie::{safe_memfd_create, safe_openat2, safe_uname},
     err::err2no,
     error,
     fs::{
@@ -597,10 +597,8 @@ fn syscall_open_handler(
         let unsafe_open_kfd = sandbox.flags.allow_unsafe_open_kfd();
         let unsafe_open_path = sandbox.flags.allow_unsafe_open_path();
         let restrict_create = !sandbox.flags.allow_unsafe_create();
-        let filter_machine_id =
-            !sandbox.flags.allow_unsafe_machine_id() && path.abs().is_machine_id();
-        let filter_proc_status =
-            !sandbox.flags.allow_unsafe_proc_pid_status() && path.abs().is_proc_pid_status();
+        let filter_machine_id = !sandbox.flags.allow_unsafe_machine_id();
+        let filter_proc_status = !sandbox.flags.allow_unsafe_proc_pid_status();
         if sandbox.flags.force_cloexec() {
             o_cloexec = true;
         }
@@ -887,7 +885,7 @@ fn syscall_open_handler(
 
                     let mut fd = result?;
 
-                    if filter_machine_id {
+                    if filter_machine_id && path.abs().is_machine_id() {
                         // SAFETY: Substitute machine-id(5) with per-Syd random boot-id.
                         //
                         // Generate machine-id(5) contents.
@@ -914,7 +912,7 @@ fn syscall_open_handler(
                         seal_memfd(&mfd)?;
 
                         fd = mfd.into();
-                    } else if filter_proc_status {
+                    } else if filter_proc_status && path.abs().is_proc_pid_status() {
                         // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
                         //
                         // Open a memory fd to be sealed for write.
@@ -929,15 +927,48 @@ fn syscall_open_handler(
                         mfd.rewind().or(Err(Errno::EIO))?;
                         seal_memfd(&mfd)?;
 
+                        fd = mfd.into();
+                    } else if path.abs().is_proc_version() {
+                        // SAFETY: Provide identical data to uname(2).
+                        //
+                        // Open a memory fd to be sealed for write.
+                        // Memory fd is exec-sealed on Linux>=6.3.
+                        let mut mfd = safe_memfd_create(c"syd-proc-version", *SAFE_MFD_FLAGS)
+                            .map(File::from)
+                            .map(BufWriter::new)?;
+
+                        // Read UtsName using uname(2) syscall.
+                        //
+                        // SAFETY: Syscall is confined with syscall argument cookies.
+                        let mut utsname = safe_uname()?;
+
+                        // Mask utsname using the sandbox.
+                        let sandbox = request.get_sandbox();
+                        sandbox.mask_uname(&mut utsname);
+                        drop(sandbox); // release the read-lock.
+
+                        // Write the utsname's Display into file,
+                        // retry on interrupts.
+                        let utsname = utsname.to_string();
+                        mfd.write_all(utsname.as_bytes())
+                            .map_err(|err| err2no(&err))?;
+                        // into_inner fails on flush errors.
+                        let mut mfd = mfd.into_inner().or(Err(Errno::EIO))?;
+
+                        // Rewind file offset and seal to deny further writes.
+                        mfd.rewind().or(Err(Errno::EIO))?;
+                        seal_memfd(&mfd)?;
+
                         fd = mfd.into();
                     }
 
                     fd
                 } else if o_mask {
                     // Cannot be O_PATH or encrypted fd!
-                    // SAFETY: If we're creating the file,
-                    // we need to change per-thread umask and
-                    // still honour POSIX ACLs.
+                    //
+                    // SAFETY:
+                    // If we're creating the file, we need to change
+                    // per-thread umask and still honour POSIX ACLs.
                     do_creat(&request, &path, flags, mode, force_umask)?
                 } else {
                     unreachable!("BUG: MUST_PATH returned a directory for open, report a bug!");
