@@ -45,7 +45,7 @@ use sendfd::RecvWithFd;
 use syd::{
     compat::epoll_ctl_safe,
     config::{PIPE_BUF, VDSO_SYSCALLS},
-    confine::confine_mdwe,
+    confine::{confine_mdwe, confine_scmp_madvise},
     cookie::{
         safe_connect, safe_pipe2, safe_shutdown, safe_socket, safer_accept4, ACCEPT4_COOKIE_ARG4,
         ACCEPT4_COOKIE_ARG5, CONNECT_COOKIE_ARG3, CONNECT_COOKIE_ARG4, CONNECT_COOKIE_ARG5,
@@ -55,12 +55,13 @@ use syd::{
         SYS_SHUTDOWN, SYS_SOCKET,
     },
     err::{SydError, SydResult},
-    fs::{closeexcept, retry_on_eintr},
+    fs::closeexcept,
     hash::SydHashMap,
     ignore_signals,
     landlock_policy::LandlockPolicy,
     parsers::sandbox::PortSet,
     path::XPathBuf,
+    retry::retry_on_eintr,
     rng::{duprand, fillrandom, randint},
     sealbox::{
         check_madvise_guard_support, madvise_guard_install, mprotect_none, mprotect_readonly, mseal,
@@ -373,7 +374,7 @@ impl<'a> Proxy<'a> {
             // can {{dr}e,}allocate memory.
             // mmap{,2} and mprotect are further confined to disable PROT_EXEC.
             "brk",
-            "madvise",
+            //"madvise", advice are confined.
             "mremap",
             "munmap",
             // can handle signals but cannot install new signal handlers.
@@ -426,6 +427,9 @@ impl<'a> Proxy<'a> {
             }
         }
 
+        // Allow safe madvise(2) advice.
+        confine_scmp_madvise(&mut filter)?;
+
         // Allow safe fcntl(2) utility calls.
         for sysname in ["fcntl", "fcntl64"] {
             let syscall = match ScmpSyscall::from_name(sysname) {
