@@ -40,8 +40,9 @@ use crate::{
     alert,
     config::*,
     confine::{
-        confine_scmp_clone, confine_scmp_clone3, extend_ioctl, scmp_add_setid_rules, scmp_arch_raw,
-        ScmpNotifReq, SydArch, Sydcall, EIDRM, EOWNERDEAD, X32_SYSCALL_BIT,
+        confine_scmp_clone, confine_scmp_clone3, confine_scmp_madvise, extend_ioctl,
+        scmp_add_setid_rules, scmp_arch_raw, ScmpNotifReq, SydArch, Sydcall, EIDRM, EOWNERDEAD,
+        X32_SYSCALL_BIT,
     },
     cookie::{
         ACCEPT4_COOKIE_ARG4, ACCEPT4_COOKIE_ARG5, BIND_COOKIE_ARG3, BIND_COOKIE_ARG4,
@@ -66,10 +67,11 @@ use crate::{
         seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
         SECCOMP_IOCTL_NOTIF_LIST, SECCOMP_IOCTL_NOTIF_SEND,
     },
-    hook::{HandlerMap, UNotifyEventRequest},
+    hook::HandlerMap,
     info,
     path::dotdot_with_nul,
     proc::proc_mmap,
+    req::UNotifyEventRequest,
     sandbox::{Flags, Sandbox, SandboxGuard},
     timer::AlarmTimer,
     workers::{aes::AesLock, UnixMap, WorkerCache, WorkerData},
@@ -414,6 +416,9 @@ impl Worker {
         // Deny clone3 with ENOSYS for compatibility.
         confine_scmp_clone3(&mut ctx)?;
 
+        // Allow safe madvise(2) advice.
+        confine_scmp_madvise(&mut ctx)?;
+
         // Deny open and {l,}stat with ENOSYS rather than KillProcess.
         // We need this because std::thread::spawn has unwanted
         // side-effects such as opening /sys/devices/system/cpu/online
